<?php

/**
 * @file
 * Implementation of Apache Solr Search Integration hooks.
 */

/**
 * Implements hook_apachesolr_query_alter().
 */
function apachesolr_proximity_apachesolr_query_alter($query) {

  // Checks whether the bias is configured for this enviromnent.
  $env_id = $query->solr('getId');
  $boost = apachesolr_environment_variable_get($env_id, 'apachesolr_proximity_boost', '8.0');

  // Parses terms and phrases. If less than two keyowrds or phrases were found,
  // do not do anything since proximity relevancy wouldn't make sense.
  $q = $query->getParam('q');
  $parsed = apachesolr_proximity_parse_query($q, $has_phrase);
  if (count($parsed) > 1) {

    // Get the operator, defaults to OR.
    $operator = $query->getParam('q.op');
    if (!$operator) {
      $operator = 'OR';
    }

    $q = apachesolr_proximity_build_query($q, $operator, $parsed, $boost);
    $query->replaceParam('q', $q);
    $query->replaceParam('defType', 'edismax');
  }
}

/**
 * Parses the keywords into terms and phrases.
 *
 * @param string $q
 *   The "q" param passed to Solr, which is the keywords entered by the user.
 * @param bool &$has_phrase
 *   Returns TRUE if the kewords contain at least one phrase.
 *
 * @return array
 *   An array of parsed terms, which are associative arrays containing:
 *   - keyword: The parsed term.
 *   - type: Either "term" or "phrase".
 */
function apachesolr_proximity_parse_query($q, &$has_phrase = NULL) {
  $has_phrase = FALSE;
  $keywords = array();

  // Returns an empty array if there were no matches.
  if (!preg_match_all(APACHESOLR_PROXIMITY_REGEX, ' ' . $q, $matches, PREG_SET_ORDER)) {
    return $keywords;
  }

  // Iterate over matches and determine if they are phrases or terms.
  foreach ($matches as $match) {
    if (0 === strpos($match[2], '"')) {
      // Phrases start with quotes.
      $has_phrase = TRUE;
      $keywords[] = array(
        'keyword' => trim($match[2], '"'),
        'type' => 'phrase',
      );
    }
    elseif (!preg_match('/^AND( NOT)?|NOT$/', $match[2])) {
      // Don't capture operators as terms.
      $keywords[] = array(
        'keyword' => $match[2],
        'type' => 'term',
      );
    }
  }

  return $keywords;
}

/**
 * Builds the proximity query to be set as the "q" parameter.
 *
 * @param string $q
 *   The "q" param passed to Solr, which is the keywords entered by the user.
 * @param string $operator
 *   The operator, either "OR" or "AND".
 * @param array $parsed
 *   The parsed keywords as returned by apachesolr_proximity_parse_query().
 * @param float $boost
 *   The boost factor for the term proximity.
 *
 * @return string
 *   The formatted proximity query.
 */
function apachesolr_proximity_build_query($q, $operator, array $parsed, $boost) {

  // Gather all terms used in the proximity query.
  $proximity_terms = array();
  foreach ($parsed as $keyword) {
    if ('term' == $keyword['type']) {
      $proximity_terms[] = $keyword['keyword'];
    }
    elseif (preg_match_all(APACHESOLR_PROXIMITY_REGEX, ' ' . $keyword['keyword'], $terms, PREG_SET_ORDER)) {
      //As per the relevancy tuning doc, we only need the first term.
      $proximity_terms[] = $terms[0][0];
    }
  }

  // Build the "q" parameter according to the relevancy cookbook.
  // @see http://wiki.apache.org/solr/SolrRelevancyCookbook#Term_Proximity
  $proximity_clause = '"' . join($proximity_terms, ' ') . '"~1000000';
  if ('OR' == $operator) {
    $proximity_query = $q . ' ' . $proximity_clause . '^' . $boost;
  }
  elseif (!$has_phrase) {
    // @todo figure out how to work with phrases for AND operator. Not too
    // critical since the OR operator is recommended to use in conjunction
    // with this module and it is the Apache Solr Search Integration module's
    // default setting.
    $proximity_query = $proximity_clause;
  }

  return $proximity_query;
}
